LIBRARY ieee;
USE ieee.std_logic_1164.all;
 
  ENTITY  TxBlock IS 
	 PORT 
	 ( 
	    cal_blk_clk		: IN STD_LOGIC ;
		pll_inclk		: IN STD_LOGIC ;
		-- pll_powerdown		: IN STD_LOGIC_VECTOR (0 DOWNTO 0); internal sig
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		-- tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);  internal sig
		--coreclkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		-- pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);  internal sig
		tx_dataout		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		reset_tx		: in  std_logic;
		The_clk			: IN STD_LOGIC ;  -- clock for reset 
		
		data0		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);		
		rdreq0		: IN STD_LOGIC ;
		wrclk0		: IN STD_LOGIC ;
		wrreq0		: IN STD_LOGIC ;
		 
		data1		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);		
		rdreq1		: IN STD_LOGIC ;
		wrclk1		: IN STD_LOGIC ;
		wrreq1		: IN STD_LOGIC ;
		 
		data2		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);		
		rdreq2		: IN STD_LOGIC ;
		wrclk2		: IN STD_LOGIC ;
		wrreq2		: IN STD_LOGIC ;
		
		clrrom       : IN STD_LOGIC;
		muxsel       : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		
		sync_data : OUT STD_LOGIC_VECTOR (127 downto 0)
	 ); 
  END TxBlock;
 
ARCHITECTURE arctx OF TxBlock IS

component tx
	PORT
	(
		cal_blk_clk		: IN STD_LOGIC ;
		pll_inclk		: IN STD_LOGIC ;
		pll_powerdown		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (127 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		coreclkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
	);
end component;

component FIFO28
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR (27 DOWNTO 0)
	);
end component;

component  TxReset
   port (
	clk             : in  std_logic;
	POWER_UP_RST_N  : in  std_logic; 						-- similar to reset signal
	pll_locked      : in  std_logic_vector (0 downto 0);
	pll_areset      : out std_logic_vector (0 downto 0);    -- similar to pll_powerdown
	tx_digitalreset : out std_logic_vector (0 downto 0);
	tx_ready        : out std_logic; -- for rx reset machine
	debug_st        : out std_logic_vector (1 downto 0) -- will be synthesized away if not used
	);
end component;



component ROM
    PORT
	(
		clk :  IN  STD_LOGIC;
		clr :  IN STD_LOGIC;
		data :  OUT  STD_LOGIC_VECTOR(127 DOWNTO 0)
	);
end component;

component FIFO128
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (127 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR (127 DOWNTO 0)
	);
end component;




-- serial wire
signal datatomux : std_logic_vector(127 downto 0)  :=  (OTHERS => '0');
signal datarom   : std_logic_vector(127 downto 0);
signal datamuxout: std_logic_vector(127 downto 0);
signal tx_coreclkout: std_logic_vector(0 downto 0);

signal ctrlmuxout : std_logic_vector(15 downto 0)  :=  (OTHERS => '0');
signal pll_locked_sig :   std_logic_vector (0 downto 0);
signal tx_digitalreset_sig		:  STD_LOGIC_VECTOR (0 DOWNTO 0);
signal pll_powerdown_sig		:  STD_LOGIC_VECTOR (0 DOWNTO 0);
signal reset_signal_nn :   STD_LOGIC ;  


BEGIN
reset_signal_nn <= NOT reset_tx;


-- tx reset state machine
	txrst_inst: TxReset PORT MAP (
		clk              => The_clk,
		POWER_UP_RST_N   => reset_signal_nn,
		pll_locked       => pll_locked_sig,
		pll_areset       => pll_powerdown_sig,
		tx_digitalreset  => tx_digitalreset_sig,
		tx_ready         => open,
		debug_st         => open
	);
	
	
	
 
-- tx
	tx_inst : tx PORT MAP (
		cal_blk_clk	 => cal_blk_clk,
		pll_powerdown	 => pll_powerdown_sig,
		pll_inclk	 => pll_inclk,		
		tx_ctrlenable	 => ctrlmuxout,
		tx_datain	 => datamuxout,
		tx_digitalreset	 => tx_digitalreset_sig,
		pll_locked	 => pll_locked_sig,
		coreclkout	 => tx_coreclkout,
		tx_dataout	 => tx_dataout(7 downto 0)
	);
	
 -- fifo28_inst
    fifo28_inst0 : FIFO28 PORT MAP (
		data    =>  data0,	 
		rdclk   =>	tx_coreclkout(0), 
		rdreq	=> rdreq0,	 
		wrclk	=> wrclk0,	 
		wrreq	=> wrreq0,	 
		q		=>  datatomux(27 downto 0)
		);
		
	fifo28_inst1 : FIFO28 PORT MAP (
		data    =>  data1,	 
		rdclk   =>	tx_coreclkout(0), 
		rdreq	=> rdreq1,	 
		wrclk	=> wrclk1,	 
		wrreq	=> wrreq1,	 
		q		=>  datatomux(55 downto 28)
		);
		
    fifo28_inst2 : FIFO28 PORT MAP (
		data    =>  data2,	 
		rdclk   =>	tx_coreclkout(0), 
		rdreq	=> rdreq2,	 
		wrclk	=> wrclk2,	 
		wrreq	=> wrreq2,	 
		q		=>  datatomux(83 downto 56)
		);
		
-- ROM inst
	rom_inst : ROM PORT MAP (
		clk => tx_coreclkout(0),  
		clr =>  clrrom, 
		data => datarom
		);
		
FIFO128_inst : FIFO128 PORT MAP (
		data	 => datatomux,
		rdclk	 => The_clk,
		rdreq	 => '1',
		wrclk	 => tx_coreclkout(0),
		wrreq	 => '1',
		q	 => sync_data
	);
		

process(muxsel,datarom,datatomux, tx_ctrlenable)
	begin
		if muxsel = "00" then
			datamuxout <= datarom;
			ctrlmuxout <= (others=>'0');
		elsif muxsel = "01" then
			datamuxout <= datatomux;
			ctrlmuxout <= tx_ctrlenable;
		else  --            0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1
			datamuxout <= X"FFBCFFBCFFBCFFBCFFBCFFBCFFBCFFBC";
			ctrlmuxout <= "0101010101010101";
		end if;
end process;
			

		
		
END arctx;
