 LIBRARY ieee;
 USE ieee.std_logic_1164.all;
 
  ENTITY  TxFIFO IS 
	 PORT 
	 ( 
	    cal_blk_clk		: IN STD_LOGIC ;
		pll_inclk		: IN STD_LOGIC ;
		pll_powerdown		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		--coreclkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
		
		--rdclk		: IN STD_LOGIC ;
		
		data0		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);		
		rdreq0		: IN STD_LOGIC ;
		wrclk0		: IN STD_LOGIC ;
		wrreq0		: IN STD_LOGIC ;
		 
		data1		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);		
		rdreq1		: IN STD_LOGIC ;
		wrclk1		: IN STD_LOGIC ;
		wrreq1		: IN STD_LOGIC ;
		 
		data2		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);		
		rdreq2		: IN STD_LOGIC ;
		wrclk2		: IN STD_LOGIC ;
		wrreq2		: IN STD_LOGIC ;
		
		clrrom       : IN STD_LOGIC;
		muxsel       : IN STD_LOGIC_VECTOR (1 DOWNTO 0)
	 ); 
 END TxFIFO;
 
ARCHITECTURE arctx OF TxFIFO IS

component tx
	PORT
	(
		cal_blk_clk		: IN STD_LOGIC ;
		pll_inclk		: IN STD_LOGIC ;
		pll_powerdown		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (127 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		coreclkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
	);
end component;

component FIFO28
	PORT
	(
		data		: IN STD_LOGIC_VECTOR (27 DOWNTO 0);
		rdclk		: IN STD_LOGIC ;
		rdreq		: IN STD_LOGIC ;
		wrclk		: IN STD_LOGIC ;
		wrreq		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR (27 DOWNTO 0)
	);
end component;

--component MUX128x2
--	PORT
--	(
--		clock		: IN STD_LOGIC ;
--		data0x		: IN STD_LOGIC_VECTOR (127 DOWNTO 0);
--		data1x		: IN STD_LOGIC_VECTOR (127 DOWNTO 0);
--		data2x		: IN STD_LOGIC_VECTOR (127 DOWNTO 0);
--		sel		    : IN STD_LOGIC_VECTOR (1 DOWNTO 0) ;
--		result		: OUT STD_LOGIC_VECTOR (127 DOWNTO 0)
--	);
--end component;

component ROM
    PORT
	(
		clk :  IN  STD_LOGIC;
		clr :  IN STD_LOGIC;
		data :  OUT  STD_LOGIC_VECTOR(127 DOWNTO 0)
	);
end component;


-- serial wire
signal datatomux : std_logic_vector(127 downto 0)  :=  (OTHERS => '0');
signal datarom   : std_logic_vector(127 downto 0);
signal datamuxout: std_logic_vector(127 downto 0);
signal tx_coreclkout: std_logic_vector(0 downto 0);

--signal ctrlmuxout : std_logic_vector(15 downto 0)  :=  (OTHERS => '0');

BEGIN

process(muxsel,datarom,datatomux)
	begin
		if muxsel = "00" then
			datamuxout <= datarom;
			-- ctrlmuxout <= (others=>'0');
		elsif muxsel = "01" then
			datamuxout <= datatomux;
			-- ctrlmuxout <= tx_ctrlenable
		else  --            0 1 0 1 0 1 0 1 0 1 0 1 0 1 0 1
			datamuxout <= X"FFBCFFBCFFBCFFBCFFBCFFBCFFBCFFBC";
			-- ctrlmuxout <= "010101"
		end if;
end process;
	
 
-- tx 2 copies
	tx_inst : tx PORT MAP (
		cal_blk_clk	 => cal_blk_clk,
		pll_powerdown	 => pll_powerdown,
		pll_inclk	 => pll_inclk,		
		tx_ctrlenable	 => ctrlmuxout,
		tx_datain	 => datamuxout,
		tx_digitalreset	 => tx_digitalreset,
		pll_locked	 => pll_locked,
		coreclkout	 => tx_coreclkout,
		tx_dataout	 => tx_dataout(7 downto 0)
	);
	
 -- fifo28_inst
    fifo28_inst0 : FIFO28 PORT MAP (
		data    =>  data0,	 
		rdclk   =>	tx_coreclkout(0), 
		rdreq	=> rdreq0,	 
		wrclk	=> wrclk0,	 
		wrreq	=> wrreq0,	 
		q		=>  datatomux(27 downto 0)
		);
		
	fifo28_inst1 : FIFO28 PORT MAP (
		data    =>  data1,	 
		rdclk   =>	tx_coreclkout(0), 
		rdreq	=> rdreq1,	 
		wrclk	=> wrclk1,	 
		wrreq	=> wrreq1,	 
		q		=>  datatomux(55 downto 28)
		);
		
    fifo28_inst2 : FIFO28 PORT MAP (
		data    =>  data2,	 
		rdclk   =>	tx_coreclkout(0), 
		rdreq	=> rdreq2,	 
		wrclk	=> wrclk2,	 
		wrreq	=> wrreq2,	 
		q		=>  datatomux(83 downto 56)
		);
		
-- ROM inst
	rom_inst : ROM PORT MAP (
		clk => tx_coreclkout(0),  
		clr =>  clrrom, 
		data => datarom
		);
		
--MUX inst
--    mux128x2_inst: MUX128x2 PORT MAP (
--		clock	=> 	 tx_coreclkout(0),
--		data0x	=> 	 datarom,	 
--		data1x	=> 	 datatomux, 	 
--		sel		=> 	 muxsel,  
--		result	=> 	datamuxout
--		);
		
		
END arctx;
